@@ -1,5 +1,5 @@
 # attributes.py - manages object attributes
-# Copyright (C) 2005, 2006, 2007, 2008, 2009 Michael Bayer mike_mp@zzzcomputing.com
+# Copyright (C) 2005, 2006, 2007, 2008, 2009, 2010 Michael Bayer mike_mp@zzzcomputing.com
 #
 # This module is part of SQLAlchemy and is released under
 # the MIT License: http://www.opensource.org/licenses/mit-license.php
@@ -28,7 +28,7 @@ _entity_info = None
 identity_equal = None
 state = None
 
-PASSIVE_NORESULT = util.symbol('PASSIVE_NORESULT')
+PASSIVE_NO_RESULT = util.symbol('PASSIVE_NO_RESULT')
 ATTR_WAS_SET = util.symbol('ATTR_WAS_SET')
 NO_VALUE = util.symbol('NO_VALUE')
 NEVER_SET = util.symbol('NEVER_SET')
@@ -43,7 +43,7 @@ PASSIVE_NO_INITIALIZE = True #util.symbol('PASSIVE_NO_INITIALIZE')
 # don't fire off any callables, but if no callables present
 # then initialize to an empty value/collection
 # this is used by backrefs.
-PASSIVE_NO_CALLABLES = util.symbol('PASSIVE_NO_CALLABLES')
+PASSIVE_NO_FETCH = util.symbol('PASSIVE_NO_FETCH')
 
 # fire callables/initialize as needed
 PASSIVE_OFF = False #util.symbol('PASSIVE_OFF')
@@ -129,7 +129,8 @@ class QueryableAttribute(interfaces.PropComparator):
         try:
             return getattr(self.comparator, key)
         except AttributeError:
-            raise AttributeError('Neither %r object nor %r object has an attribute %r' % (
+            raise AttributeError(
+                    'Neither %r object nor %r object has an attribute %r' % (
                     type(self).__name__, 
                     type(self.comparator).__name__, 
                     key)
@@ -159,7 +160,7 @@ class InstrumentedAttribute(QueryableAttribute):
 
 class _ProxyImpl(object):
     accepts_scalar_loader = False
-    dont_expire_missing = False
+    expire_missing = True
     
     def __init__(self, key):
         self.key = key
@@ -212,7 +213,8 @@ def proxied_attribute_factory(descriptor):
                 try:
                     return getattr(self._comparator, attribute)
                 except AttributeError:
-                    raise AttributeError('Neither %r object nor %r object has an attribute %r' % (
+                    raise AttributeError(
+                            'Neither %r object nor %r object has an attribute %r' % (
                             type(descriptor).__name__, 
                             type(self._comparator).__name__, 
                             attribute)
@@ -230,8 +232,8 @@ class AttributeImpl(object):
 
     def __init__(self, class_, key,
                     callable_, trackparent=False, extension=None,
-                    compare_function=None, active_history=False, parent_token=None, 
-                    dont_expire_missing=False,
+                    compare_function=None, active_history=False, 
+                    parent_token=None, expire_missing=True,
                     **kwargs):
         """Construct an AttributeImpl.
 
@@ -262,7 +264,6 @@ class AttributeImpl(object):
         active_history
           indicates that get_history() should always return the "old" value,
           even if it means executing a lazy callable upon attribute change.
-          This flag is set to True if any extensions are present.
 
         parent_token
           Usually references the MapperProperty, used as a key for
@@ -270,8 +271,8 @@ class AttributeImpl(object):
           Allows multiple AttributeImpls to all match a single 
           owner attribute.
           
-        dont_expire_missing
-          if True, don't add an "expiry" callable to this attribute
+        expire_missing
+          if False, don't add an "expiry" callable to this attribute
           during state.expire_attributes(None), if no value is present 
           for this key.
           
@@ -286,11 +287,16 @@ class AttributeImpl(object):
         else:
             self.is_equal = compare_function
         self.extensions = util.to_list(extension or [])
+        for e in self.extensions:
+            if e.active_history:
+                active_history = True
+                break
         self.active_history = active_history
-        self.dont_expire_missing = dont_expire_missing
+        self.expire_missing = expire_missing
         
     def hasparent(self, state, optimistic=False):
-        """Return the boolean value of a `hasparent` flag attached to the given item.
+        """Return the boolean value of a `hasparent` flag attached to 
+        the given state.
 
         The `optimistic` flag determines what the default return value
         should be if no `hasparent` flag can be located.
@@ -329,10 +335,7 @@ class AttributeImpl(object):
         ``InstrumentedAttribute`` constructor.
 
         """
-        if callable_ is None:
-            self.initialize(state)
-        else:
-            state.callables[self.key] = callable_
+        state.callables[self.key] = callable_
 
     def get_history(self, state, dict_, passive=PASSIVE_OFF):
         raise NotImplementedError()
@@ -346,7 +349,7 @@ class AttributeImpl(object):
             return None
 
     def initialize(self, state, dict_):
-        """Initialize this attribute on the given object instance with an empty value."""
+        """Initialize the given state's attribute with an empty value."""
 
         dict_[self.key] = None
         return None
@@ -365,30 +368,32 @@ class AttributeImpl(object):
             # if no history, check for lazy callables, etc.
             if state.committed_state.get(self.key, NEVER_SET) is NEVER_SET:
                 if passive is PASSIVE_NO_INITIALIZE:
-                    return PASSIVE_NORESULT
+                    return PASSIVE_NO_RESULT
                     
                 callable_ = self._get_callable(state)
                 if callable_ is not None:
-                    if passive is not PASSIVE_OFF:
-                        return PASSIVE_NORESULT
-                    value = callable_()
-                    if value is not ATTR_WAS_SET:
+                    #if passive is not PASSIVE_OFF:
+                    #    return PASSIVE_NO_RESULT
+                    value = callable_(passive=passive)
+                    if value is PASSIVE_NO_RESULT:
+                        return value
+                    elif value is not ATTR_WAS_SET:
                         return self.set_committed_value(state, dict_, value)
                     else:
                         if self.key not in dict_:
-                            return self.get(state, passive=passive)
+                            return self.get(state, dict_, passive=passive)
                         return dict_[self.key]
 
             # Return a new, empty value
             return self.initialize(state, dict_)
 
     def append(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
-        self.set(state, dict_, value, initiator)
+        self.set(state, dict_, value, initiator, passive=passive)
 
     def remove(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
-        self.set(state, dict_, None, initiator)
+        self.set(state, dict_, None, initiator, passive=passive)
 
-    def set(self, state, dict_, value, initiator):
+    def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
         raise NotImplementedError()
 
     def get_committed_value(self, state, dict_, passive=PASSIVE_OFF):
@@ -421,34 +426,32 @@ class ScalarAttributeImpl(AttributeImpl):
     def delete(self, state, dict_):
 
         # TODO: catch key errors, convert to attributeerror?
-        if self.active_history or self.extensions:
+        if self.active_history:
             old = self.get(state, dict_)
         else:
             old = dict_.get(self.key, NO_VALUE)
 
-        state.modified_event(dict_, self, False, old)
-
         if self.extensions:
             self.fire_remove_event(state, dict_, old, None)
+        state.modified_event(dict_, self, False, old)
         del dict_[self.key]
 
     def get_history(self, state, dict_, passive=PASSIVE_OFF):
         return History.from_attribute(
             self, state, dict_.get(self.key, NO_VALUE))
 
-    def set(self, state, dict_, value, initiator):
+    def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
         if initiator is self:
             return
 
-        if self.active_history or self.extensions:
+        if self.active_history:
             old = self.get(state, dict_)
         else:
             old = dict_.get(self.key, NO_VALUE)
 
-        state.modified_event(dict_, self, False, old)
-
         if self.extensions:
             value = self.fire_replace_event(state, dict_, value, old, initiator)
+        state.modified_event(dict_, self, False, old)
         dict_[self.key] = value
 
     def fire_replace_event(self, state, dict_, value, previous, initiator):
@@ -475,11 +478,16 @@ class MutableScalarAttributeImpl(ScalarAttributeImpl):
     def __init__(self, class_, key, callable_,
                     class_manager, copy_function=None,
                     compare_function=None, **kwargs):
-        super(ScalarAttributeImpl, self).__init__(class_, key, callable_,
-                                compare_function=compare_function, **kwargs)
+        super(ScalarAttributeImpl, self).__init__(
+                                            class_, 
+                                            key, 
+                                            callable_,
+                                            compare_function=compare_function, 
+                                            **kwargs)
         class_manager.mutable_attributes.add(key)
         if copy_function is None:
-            raise sa_exc.ArgumentError("MutableScalarAttributeImpl requires a copy function")
+            raise sa_exc.ArgumentError(
+                        "MutableScalarAttributeImpl requires a copy function")
         self.copy = copy_function
 
     def get_history(self, state, dict_, passive=PASSIVE_OFF):
@@ -491,17 +499,14 @@ class MutableScalarAttributeImpl(ScalarAttributeImpl):
         return History.from_attribute(
             self, state, v)
 
-    def commit_to_state(self, state, dict_, dest):
-        dest[self.key] = self.copy(dict_[self.key])
-
     def check_mutable_modified(self, state, dict_):
-        (added, unchanged, deleted) = self.get_history(state, dict_, passive=PASSIVE_NO_INITIALIZE)
-        return bool(added or deleted)
+        a, u, d = self.get_history(state, dict_)
+        return bool(a or d)
 
     def get(self, state, dict_, passive=PASSIVE_OFF):
         if self.key not in state.mutable_dict:
             ret = ScalarAttributeImpl.get(self, state, dict_, passive=passive)
-            if ret is not PASSIVE_NORESULT:
+            if ret is not PASSIVE_NO_RESULT:
                 state.mutable_dict[self.key] = ret
             return ret
         else:
@@ -511,25 +516,25 @@ class MutableScalarAttributeImpl(ScalarAttributeImpl):
         ScalarAttributeImpl.delete(self, state, dict_)
         state.mutable_dict.pop(self.key)
 
-    def set(self, state, dict_, value, initiator):
+    def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
         if initiator is self:
             return
 
-        state.modified_event(dict_, self, True, NEVER_SET)
-        
         if self.extensions:
             old = self.get(state, dict_)
             value = self.fire_replace_event(state, dict_, value, old, initiator)
-            dict_[self.key] = value
-        else:
-            dict_[self.key] = value
+
+        state.modified_event(dict_, self, True, NEVER_SET)
+        dict_[self.key] = value
         state.mutable_dict[self.key] = value
 
 
 class ScalarObjectAttributeImpl(ScalarAttributeImpl):
-    """represents a scalar-holding InstrumentedAttribute, where the target object is also instrumented.
+    """represents a scalar-holding InstrumentedAttribute, 
+       where the target object is also instrumented.
 
-    Adds events to delete/set operations.
+       Adds events to delete/set operations.
+       
     """
 
     accepts_scalar_loader = False
@@ -538,9 +543,14 @@ class ScalarObjectAttributeImpl(ScalarAttributeImpl):
     def __init__(self, class_, key, callable_, 
                     trackparent=False, extension=None, copy_function=None,
                     compare_function=None, **kwargs):
-        super(ScalarObjectAttributeImpl, self).__init__(class_, key,
-          callable_, trackparent=trackparent, extension=extension,
-          compare_function=compare_function, **kwargs)
+        super(ScalarObjectAttributeImpl, self).__init__(
+                                                class_, 
+                                                key,
+                                                callable_, 
+                                                trackparent=trackparent, 
+                                                extension=extension,
+                                                compare_function=compare_function, 
+                                                **kwargs)
         if compare_function is None:
             self.is_equal = identity_equal
 
@@ -554,12 +564,12 @@ class ScalarObjectAttributeImpl(ScalarAttributeImpl):
             return History.from_attribute(self, state, dict_[self.key])
         else:
             current = self.get(state, dict_, passive=passive)
-            if current is PASSIVE_NORESULT:
+            if current is PASSIVE_NO_RESULT:
                 return HISTORY_BLANK
             else:
                 return History.from_attribute(self, state, current)
 
-    def set(self, state, dict_, value, initiator):
+    def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
         """Set a value on the given InstanceState.
 
         `initiator` is the ``InstrumentedAttribute`` that initiated the
@@ -570,30 +580,35 @@ class ScalarObjectAttributeImpl(ScalarAttributeImpl):
         if initiator is self:
             return
 
-        # may want to add options to allow the get() here to be passive
-        old = self.get(state, dict_)
+        if self.active_history:
+            old = self.get(state, dict_)
+        else:
+            old = self.get(state, dict_, passive=PASSIVE_NO_FETCH)
+             
         value = self.fire_replace_event(state, dict_, value, old, initiator)
         dict_[self.key] = value
 
     def fire_remove_event(self, state, dict_, value, initiator):
-        state.modified_event(dict_, self, False, value)
-
         if self.trackparent and value is not None:
             self.sethasparent(instance_state(value), False)
 
         for ext in self.extensions:
             ext.remove(state, value, initiator or self)
 
-    def fire_replace_event(self, state, dict_, value, previous, initiator):
-        state.modified_event(dict_, self, False, previous)
+        state.modified_event(dict_, self, False, value)
 
+    def fire_replace_event(self, state, dict_, value, previous, initiator):
         if self.trackparent:
-            if previous is not value and previous is not None:
+            if (previous is not value and
+                previous is not None and
+                previous is not PASSIVE_NO_RESULT):
                 self.sethasparent(instance_state(previous), False)
 
         for ext in self.extensions:
             value = ext.set(state, value, previous, initiator or self)
 
+        state.modified_event(dict_, self, False, previous)
+
         if self.trackparent:
             if value is not None:
                 self.sethasparent(instance_state(value), True)
@@ -618,34 +633,36 @@ class CollectionAttributeImpl(AttributeImpl):
     def __init__(self, class_, key, callable_, 
                     typecallable=None, trackparent=False, extension=None,
                     copy_function=None, compare_function=None, **kwargs):
-        super(CollectionAttributeImpl, self).__init__(class_, key, callable_, trackparent=trackparent,
-          extension=extension, compare_function=compare_function, **kwargs)
+        super(CollectionAttributeImpl, self).__init__(
+                                                class_, 
+                                                key, 
+                                                callable_, 
+                                                trackparent=trackparent,
+                                                extension=extension,
+                                                compare_function=compare_function, 
+                                                **kwargs)
 
         if copy_function is None:
             copy_function = self.__copy
         self.copy = copy_function
-
         self.collection_factory = typecallable
-        # may be removed in 0.5:
-        self.collection_interface = \
-          util.duck_type_collection(self.collection_factory())
 
     def __copy(self, item):
         return [y for y in list(collections.collection_adapter(item))]
 
     def get_history(self, state, dict_, passive=PASSIVE_OFF):
         current = self.get(state, dict_, passive=passive)
-        if current is PASSIVE_NORESULT:
+        if current is PASSIVE_NO_RESULT:
             return HISTORY_BLANK
         else:
             return History.from_attribute(self, state, current)
 
     def fire_append_event(self, state, dict_, value, initiator):
-        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
-
         for ext in self.extensions:
             value = ext.append(state, value, initiator or self)
 
+        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
+
         if self.trackparent and value is not None:
             self.sethasparent(instance_state(value), True)
 
@@ -655,14 +672,14 @@ class CollectionAttributeImpl(AttributeImpl):
         state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
 
     def fire_remove_event(self, state, dict_, value, initiator):
-        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
-
         if self.trackparent and value is not None:
             self.sethasparent(instance_state(value), False)
 
         for ext in self.extensions:
             ext.remove(state, value, initiator or self)
 
+        state.modified_event(dict_, self, True, NEVER_SET, passive=PASSIVE_NO_INITIALIZE)
+
     def delete(self, state, dict_):
         if self.key not in dict_:
             return
@@ -690,8 +707,9 @@ class CollectionAttributeImpl(AttributeImpl):
             return
 
         collection = self.get_collection(state, dict_, passive=passive)
-        if collection is PASSIVE_NORESULT:
+        if collection is PASSIVE_NO_RESULT:
             value = self.fire_append_event(state, dict_, value, initiator)
+            assert self.key not in dict_, "Collection was loaded during event handling."
             state.get_pending(self.key).append(value)
         else:
             collection.append_with_event(value, initiator)
@@ -701,13 +719,14 @@ class CollectionAttributeImpl(AttributeImpl):
             return
 
         collection = self.get_collection(state, state.dict, passive=passive)
-        if collection is PASSIVE_NORESULT:
+        if collection is PASSIVE_NO_RESULT:
             self.fire_remove_event(state, dict_, value, initiator)
+            assert self.key not in dict_, "Collection was loaded during event handling."
             state.get_pending(self.key).remove(value)
         else:
             collection.remove_with_event(value, initiator)
 
-    def set(self, state, dict_, value, initiator):
+    def set(self, state, dict_, value, initiator, passive=PASSIVE_OFF):
         """Set a value on the given object.
 
         `initiator` is the ``InstrumentedAttribute`` that initiated the
@@ -795,7 +814,7 @@ class CollectionAttributeImpl(AttributeImpl):
         """
         if user_data is None:
             user_data = self.get(state, dict_, passive=passive)
-            if user_data is PASSIVE_NORESULT:
+            if user_data is PASSIVE_NO_RESULT:
                 return user_data
 
         return getattr(user_data, '_sa_adapter')
@@ -808,48 +827,70 @@ class GenericBackrefExtension(interfaces.AttributeExtension):
     are two objects which contain scalar references to each other.
 
     """
+    
+    active_history = False
+    
     def __init__(self, key):
         self.key = key
 
     def set(self, state, child, oldchild, initiator):
         if oldchild is child:
             return child
-        if oldchild is not None:
+            
+        if oldchild is not None and oldchild is not PASSIVE_NO_RESULT:
             # With lazy=None, there's no guarantee that the full collection is
             # present when updating via a backref.
             old_state, old_dict = instance_state(oldchild), instance_dict(oldchild)
             impl = old_state.get_impl(self.key)
             try:
-                impl.remove(old_state, old_dict, state.obj(), initiator, passive=PASSIVE_NO_CALLABLES)
+                impl.remove(old_state, 
+                            old_dict, 
+                            state.obj(), 
+                            initiator, passive=PASSIVE_NO_FETCH)
             except (ValueError, KeyError, IndexError):
                 pass
+                
         if child is not None:
-            new_state,  new_dict = instance_state(child), instance_dict(child)
-            new_state.get_impl(self.key).append(new_state, new_dict, state.obj(), initiator, passive=PASSIVE_NO_CALLABLES)
+            child_state, child_dict = instance_state(child), instance_dict(child)
+            child_state.get_impl(self.key).append(
+                                            child_state, 
+                                            child_dict, 
+                                            state.obj(), 
+                                            initiator, passive=PASSIVE_NO_FETCH)
         return child
 
     def append(self, state, child, initiator):
         child_state, child_dict = instance_state(child), instance_dict(child)
-        child_state.get_impl(self.key).append(child_state, child_dict, state.obj(), initiator, passive=PASSIVE_NO_CALLABLES)
+        child_state.get_impl(self.key).append(
+                                            child_state, 
+                                            child_dict, 
+                                            state.obj(), 
+                                            initiator, passive=PASSIVE_NO_FETCH)
         return child
 
     def remove(self, state, child, initiator):
         if child is not None:
             child_state, child_dict = instance_state(child), instance_dict(child)
-            child_state.get_impl(self.key).remove(child_state, child_dict, state.obj(), initiator, passive=PASSIVE_NO_CALLABLES)
+            child_state.get_impl(self.key).remove(
+                                            child_state, 
+                                            child_dict, 
+                                            state.obj(), 
+                                            initiator, passive=PASSIVE_NO_FETCH)
 
 
 class Events(object):
     def __init__(self):
         self.original_init = object.__init__
+        # Initialize to tuples instead of lists to minimize the memory 
+        # footprint
         self.on_init = ()
         self.on_init_failure = ()
         self.on_load = ()
         self.on_resurrect = ()
 
-    def run(self, event, *args, **kwargs):
+    def run(self, event, *args):
         for fn in getattr(self, event):
-            fn(*args, **kwargs)
+            fn(*args)
 
     def add_listener(self, event, listener):
         # not thread safe... problem?  mb: nope
@@ -944,9 +985,6 @@ class ClassManager(dict):
             self.new_init = None
     
     def _create_instance_state(self, instance):
-        global state
-        if state is None:
-            from sqlalchemy.orm import state
         if self.mutable_attributes:
             return state.MutableAttrInstanceState(instance, self)
         else:
@@ -973,9 +1011,8 @@ class ClassManager(dict):
             self.local_attrs[key] = inst
             self.install_descriptor(key, inst)
         self[key] = inst
+        
         for cls in self.class_.__subclasses__():
-            if isinstance(cls, types.ClassType):
-                continue
             manager = self._subclass_manager(cls)
             manager.instrument_attribute(key, inst, True)
 
@@ -995,8 +1032,6 @@ class ClassManager(dict):
         if key in self.mutable_attributes:
             self.mutable_attributes.remove(key)
         for cls in self.class_.__subclasses__():
-            if isinstance(cls, types.ClassType):
-                continue
             manager = self._subclass_manager(cls)
             manager.uninstrument_attribute(key, True)
 
@@ -1190,10 +1225,11 @@ class _ClassInstrumentationAdapter(ClassManager):
     def has_state(self, instance):
         try:
             state = self._get_state(instance)
-            return True
         except exc.NO_STATE:
             return False
-
+        else:
+            return True
+            
     def state_getter(self):
         return self._get_state
 
@@ -1219,25 +1255,41 @@ class History(tuple):
     def __nonzero__(self):
         return self != HISTORY_BLANK
     
+    def empty(self):
+        return not bool(
+                        (self.added or self.deleted)
+                        or self.unchanged and self.unchanged != [None]
+                    ) 
+        
     def sum(self):
-        return self.added + self.unchanged + self.deleted
+        return (self.added or []) +\
+                (self.unchanged or []) +\
+                (self.deleted or [])
     
     def non_deleted(self):
-        return self.added + self.unchanged
+        return (self.added or []) +\
+                (self.unchanged or [])
     
     def non_added(self):
-        return self.unchanged + self.deleted
+        return (self.unchanged or []) +\
+                (self.deleted or [])
     
     def has_changes(self):
         return bool(self.added or self.deleted)
         
     def as_state(self):
         return History(
-            [c is not None and instance_state(c) or None for c in self.added],
-            [c is not None and instance_state(c) or None for c in self.unchanged],
-            [c is not None and instance_state(c) or None for c in self.deleted],
-        )
-    
+            [(c is not None and c is not PASSIVE_NO_RESULT)
+             and instance_state(c) or None
+             for c in self.added],
+            [(c is not None and c is not PASSIVE_NO_RESULT)
+             and instance_state(c) or None
+             for c in self.unchanged],
+            [(c is not None and c is not PASSIVE_NO_RESULT)
+             and instance_state(c) or None
+             for c in self.deleted],
+            )
+        
     @classmethod
     def from_attribute(cls, attribute, state, current):
         original = state.committed_state.get(attribute.key, NEVER_SET)
@@ -1260,7 +1312,9 @@ class History(tuple):
                 )
         else:
             if current is NO_VALUE:
-                if original not in [None, NEVER_SET, NO_VALUE]:
+                if (original is not None and
+                    original is not NEVER_SET and
+                    original is not NO_VALUE):
                     deleted = [original]
                 else:
                     deleted = ()
@@ -1280,11 +1334,6 @@ class History(tuple):
 
 HISTORY_BLANK = History(None, None, None)
 
-def _conditional_instance_state(obj):
-    if not isinstance(obj, state.InstanceState):
-        obj = instance_state(obj)
-    return obj
-        
 def get_history(obj, key, **kwargs):
     """Return a History record for the given object and attribute key.
     
@@ -1293,7 +1342,7 @@ def get_history(obj, key, **kwargs):
     this usage is deprecated.
     
     """
-    return get_state_history(_conditional_instance_state(obj), key, **kwargs)
+    return get_state_history(instance_state(obj), key, **kwargs)
 
 def get_state_history(state, key, **kwargs):
     return state.get_history(key, **kwargs)
@@ -1321,12 +1370,12 @@ def unregister_class(class_):
     instrumentation_registry.unregister(class_)
 
 def register_attribute(class_, key, **kw):
-
     proxy_property = kw.pop('proxy_property', None)
     
     comparator = kw.pop('comparator', None)
     parententity = kw.pop('parententity', None)
-    register_descriptor(class_, key, proxy_property, comparator, parententity)
+    doc = kw.pop('doc', None)
+    register_descriptor(class_, key, proxy_property, comparator, parententity, doc=doc)
     if not proxy_property:
         register_attribute_impl(class_, key, **kw)
     
@@ -1360,7 +1409,8 @@ def register_attribute_impl(class_, key,
     
     manager.post_configure_attribute(key)
     
-def register_descriptor(class_, key, proxy_property=None, comparator=None, parententity=None, property_=None):
+def register_descriptor(class_, key, proxy_property=None, comparator=None, 
+                                parententity=None, property_=None, doc=None):
     manager = manager_of_class(class_)
 
     if proxy_property:
@@ -1368,7 +1418,9 @@ def register_descriptor(class_, key, proxy_property=None, comparator=None, paren
         descriptor = proxy_type(key, proxy_property, comparator, parententity)
     else:
         descriptor = InstrumentedAttribute(key, comparator=comparator, parententity=parententity)
-
+    
+    descriptor.__doc__ = doc
+        
     manager.instrument_attribute(key, descriptor)
 
 def unregister_attribute(class_, key):
@@ -1391,7 +1443,7 @@ def init_collection(obj, key):
     this usage is deprecated.
     
     """
-    state = _conditional_instance_state(obj)
+    state = instance_state(obj)
     dict_ = state.dict
     return init_state_collection(state, dict_, key)
     
@@ -1418,7 +1470,7 @@ def set_committed_value(instance, key, value):
     
     """
     state, dict_ = instance_state(instance), instance_dict(instance)
-    state.get_impl(key).set_committed_value(state, dict_, key, value)
+    state.get_impl(key).set_committed_value(state, dict_, value)
     
 def set_attribute(instance, key, value):
     """Set the value of an attribute, firing history events.
@@ -1470,7 +1522,13 @@ def is_instrumented(instance, key):
     return manager_of_class(instance.__class__).is_instrumented(key, search=True)
 
 class InstrumentationRegistry(object):
-    """Private instrumentation registration singleton."""
+    """Private instrumentation registration singleton.
+    
+    All classes are routed through this registry 
+    when first instrumented, however the InstrumentationRegistry
+    is not actually needed unless custom ClassManagers are in use.
+    
+    """
 
     _manager_finders = weakref.WeakKeyDictionary()
     _state_finders = util.WeakIdentityMapping()
@@ -1501,6 +1559,9 @@ class InstrumentationRegistry(object):
             manager = _ClassInstrumentationAdapter(class_, manager)
             
         if factory != ClassManager and not self._extended:
+            # somebody invoked a custom ClassManager.
+            # reinstall global "getter" functions with the more 
+            # expensive ones.
             self._extended = True
             _install_lookup_strategy(self)
         
@@ -1539,6 +1600,7 @@ class InstrumentationRegistry(object):
         return factories
 
     def manager_of_class(self, cls):
+        # this is only called when alternate instrumentation has been established
         if cls is None:
             return None
         try:
@@ -1574,7 +1636,9 @@ class InstrumentationRegistry(object):
             del self._manager_finders[class_]
             del self._state_finders[class_]
             del self._dict_finders[class_]
-
+        if ClassManager.MANAGER_ATTR in class_.__dict__:
+            delattr(class_, ClassManager.MANAGER_ATTR)
+        
 instrumentation_registry = InstrumentationRegistry()
 
 def _install_lookup_strategy(implementation):
@@ -1587,16 +1651,23 @@ def _install_lookup_strategy(implementation):
     and unit tests specific to this behavior.
     
     """
-    global instance_state, instance_dict
+    global instance_state, instance_dict, manager_of_class
     if implementation is util.symbol('native'):
         instance_state = attrgetter(ClassManager.STATE_ATTR)
         instance_dict = attrgetter("__dict__")
+        def manager_of_class(cls):
+            return cls.__dict__.get(ClassManager.MANAGER_ATTR, None)
     else:
         instance_state = instrumentation_registry.state_of
         instance_dict = instrumentation_registry.dict_of
+        manager_of_class = instrumentation_registry.manager_of_class
         
-manager_of_class = instrumentation_registry.manager_of_class
 _create_manager_for_cls = instrumentation_registry.create_manager_for_cls
+
+# Install default "lookup" strategies.  These are basically
+# very fast attrgetters for key attributes.
+# When a custom ClassManager is installed, more expensive per-class
+# strategies are copied over these.
 _install_lookup_strategy(util.symbol('native'))
 
 def find_native_user_instrumentation_hook(cls):
@@ -1628,8 +1699,12 @@ def __init__(%(apply_pos)s):
     func_vars = util.format_argspec_init(original__init__, grouped=False)
     func_text = func_body % func_vars
 
+    # Py3K
+    #func_defaults = getattr(original__init__, '__defaults__', None)
+    # Py2K
     func = getattr(original__init__, 'im_func', original__init__)
     func_defaults = getattr(func, 'func_defaults', None)
+    # end Py2K
 
     env = locals().copy()
     exec func_text in env
